package sun.management;

import javax.management.ObjectName;
import java.util.Arrays;
import java.lang.management.ThreadInfo;
import com.sun.management.ThreadMXBean;

class ThreadImpl implements ThreadMXBean
{
    private final VMManagement jvm;
    private boolean contentionMonitoringEnabled;
    private boolean cpuTimeEnabled;
    private boolean allocatedMemoryEnabled;
    
    ThreadImpl(VMManagement jvm) {
        this.contentionMonitoringEnabled = false;
        this.jvm = jvm;
        this.cpuTimeEnabled = this.jvm.isThreadCpuTimeEnabled();
        this.allocatedMemoryEnabled = this.jvm.isThreadAllocatedMemoryEnabled();
    }
    
    @Override
    public int getThreadCount() {
        return this.jvm.getLiveThreadCount();
    }
    
    @Override
    public int getPeakThreadCount() {
        return this.jvm.getPeakThreadCount();
    }
    
    @Override
    public long getTotalStartedThreadCount() {
        return this.jvm.getTotalThreadCount();
    }
    
    @Override
    public int getDaemonThreadCount() {
        return this.jvm.getDaemonThreadCount();
    }
    
    @Override
    public boolean isThreadContentionMonitoringSupported() {
        return this.jvm.isThreadContentionMonitoringSupported();
    }
    
    @Override
    public synchronized boolean isThreadContentionMonitoringEnabled() {
        if (!this.isThreadContentionMonitoringSupported()) {
            throw new UnsupportedOperationException("Thread contention monitoring is not supported.");
        }
        return this.contentionMonitoringEnabled;
    }
    
    @Override
    public boolean isThreadCpuTimeSupported() {
        return this.jvm.isOtherThreadCpuTimeSupported();
    }
    
    @Override
    public boolean isCurrentThreadCpuTimeSupported() {
        return this.jvm.isCurrentThreadCpuTimeSupported();
    }
    
    @Override
    public boolean isThreadAllocatedMemorySupported() {
        return this.jvm.isThreadAllocatedMemorySupported();
    }
    
    @Override
    public boolean isThreadCpuTimeEnabled() {
        if (!this.isThreadCpuTimeSupported() && !this.isCurrentThreadCpuTimeSupported()) {
            throw new UnsupportedOperationException("Thread CPU time measurement is not supported");
        }
        return this.cpuTimeEnabled;
    }
    
    @Override
    public boolean isThreadAllocatedMemoryEnabled() {
        if (!this.isThreadAllocatedMemorySupported()) {
            throw new UnsupportedOperationException("Thread allocated memory measurement is not supported");
        }
        return this.allocatedMemoryEnabled;
    }
    
    @Override
    public long[] getAllThreadIds() {
        Util.checkMonitorAccess();
        Thread[] threads = getThreads();
        int length = threads.length;
        long[] array = new long[length];
        for (int i = 0; i < length; ++i) {
            array[i] = threads[i].getId();
        }
        return array;
    }
    
    @Override
    public ThreadInfo getThreadInfo(long n) {
        return this.getThreadInfo(new long[] { n }, 0)[0];
    }
    
    @Override
    public ThreadInfo getThreadInfo(long n, int n2) {
        return this.getThreadInfo(new long[] { n }, n2)[0];
    }
    
    @Override
    public ThreadInfo[] getThreadInfo(long[] array) {
        return this.getThreadInfo(array, 0);
    }
    
    private void verifyThreadIds(long[] array) {
        if (array == null) {
            throw new NullPointerException("Null ids parameter.");
        }
        int i = 0;
        while (i < array.length) {
            if (array[i] <= 0L) {
                throw new IllegalArgumentException("Invalid thread ID parameter: " + array[i]);
            }
            ++i;
        }
        return;
    }
    
    @Override
    public ThreadInfo[] getThreadInfo(long[] array, int i) {
        this.verifyThreadIds(array);
        if (i < 0) {
            throw new IllegalArgumentException("Invalid maxDepth parameter: " + i);
        }
        if (array.length == 0) {
            return new ThreadInfo[0];
        }
        Util.checkMonitorAccess();
        ThreadInfo[] array2 = new ThreadInfo[array.length];
        if (i == Integer.MAX_VALUE) {
            getThreadInfo1(array, -1, array2);
            return array2;
        }
        getThreadInfo1(array, i, array2);
        return array2;
    }
    
    @Override
    public void setThreadContentionMonitoringEnabled(boolean b) {
        if (!this.isThreadContentionMonitoringSupported()) {
            throw new UnsupportedOperationException("Thread contention monitoring is not supported");
        }
        Util.checkControlAccess();
        synchronized (this) {
            if (this.contentionMonitoringEnabled != b) {
                if (b) {
                    resetContentionTimes0(0L);
                }
                setThreadContentionMonitoringEnabled0(b);
                this.contentionMonitoringEnabled = b;
            }
        }
        return;
    }
    
    private boolean verifyCurrentThreadCpuTime() {
        if (!this.isCurrentThreadCpuTimeSupported()) {
            throw new UnsupportedOperationException("Current thread CPU time measurement is not supported.");
        }
        return this.isThreadCpuTimeEnabled();
    }
    
    @Override
    public long getCurrentThreadCpuTime() {
        if (this.verifyCurrentThreadCpuTime()) {
            return getThreadTotalCpuTime0(0L);
        }
        return -1L;
    }
    
    @Override
    public long getThreadCpuTime(long n) {
        return this.getThreadCpuTime(new long[] { n })[0];
    }
    
    private boolean verifyThreadCpuTime(long[] array) {
        this.verifyThreadIds(array);
        if (!this.isThreadCpuTimeSupported() && !this.isCurrentThreadCpuTimeSupported()) {
            throw new UnsupportedOperationException("Thread CPU time measurement is not supported.");
        }
        if (!this.isThreadCpuTimeSupported()) {
            int i = 0;
            while (i < array.length) {
                if (array[i] != Thread.currentThread().getId()) {
                    throw new UnsupportedOperationException("Thread CPU time measurement is only supported for the current thread.");
                }
                ++i;
            }
        }
        return this.isThreadCpuTimeEnabled();
    }
    
    @Override
    public long[] getThreadCpuTime(long[] array) {
        int verifyThreadCpuTime = this.verifyThreadCpuTime(array);
        int length = array.length;
        long[] a = new long[length];
        Arrays.fill(a, -1L);
        if (!(verifyThreadCpuTime == 0)) {
            if (length == 1) {
                long n = array[0];
                if (n == Thread.currentThread().getId()) {
                    n = 0L;
                }
                a[0] = getThreadTotalCpuTime0(n);
            }
            else {
                getThreadTotalCpuTime1(array, a);
            }
        }
        return a;
    }
    
    @Override
    public long getCurrentThreadUserTime() {
        if (this.verifyCurrentThreadCpuTime()) {
            return getThreadUserCpuTime0(0L);
        }
        return -1L;
    }
    
    @Override
    public long getThreadUserTime(long n) {
        return this.getThreadUserTime(new long[] { n })[0];
    }
    
    @Override
    public long[] getThreadUserTime(long[] array) {
        int verifyThreadCpuTime = this.verifyThreadCpuTime(array);
        int length = array.length;
        long[] a = new long[length];
        Arrays.fill(a, -1L);
        if (!(verifyThreadCpuTime == 0)) {
            if (length == 1) {
                long n = array[0];
                if (n == Thread.currentThread().getId()) {
                    n = 0L;
                }
                a[0] = getThreadUserCpuTime0(n);
            }
            else {
                getThreadUserCpuTime1(array, a);
            }
        }
        return a;
    }
    
    @Override
    public void setThreadCpuTimeEnabled(boolean b) {
        if (!this.isThreadCpuTimeSupported() && !this.isCurrentThreadCpuTimeSupported()) {
            throw new UnsupportedOperationException("Thread CPU time measurement is not supported");
        }
        Util.checkControlAccess();
        synchronized (this) {
            if (this.cpuTimeEnabled != b) {
                setThreadCpuTimeEnabled0(b);
                this.cpuTimeEnabled = b;
            }
        }
        return;
    }
    
    @Override
    public long getThreadAllocatedBytes(long n) {
        return this.getThreadAllocatedBytes(new long[] { n })[0];
    }
    
    private boolean verifyThreadAllocatedMemory(long[] array) {
        this.verifyThreadIds(array);
        if (!this.isThreadAllocatedMemorySupported()) {
            throw new UnsupportedOperationException("Thread allocated memory measurement is not supported.");
        }
        return this.isThreadAllocatedMemoryEnabled();
    }
    
    @Override
    public long[] getThreadAllocatedBytes(long[] array) {
        int verifyThreadAllocatedMemory = this.verifyThreadAllocatedMemory(array);
        long[] a = new long[array.length];
        Arrays.fill(a, -1L);
        if (!(verifyThreadAllocatedMemory == 0)) {
            getThreadAllocatedMemory1(array, a);
            return a;
        }
        return a;
    }
    
    @Override
    public void setThreadAllocatedMemoryEnabled(boolean b) {
        if (!this.isThreadAllocatedMemorySupported()) {
            throw new UnsupportedOperationException("Thread allocated memory measurement is not supported.");
        }
        Util.checkControlAccess();
        synchronized (this) {
            if (this.allocatedMemoryEnabled != b) {
                setThreadAllocatedMemoryEnabled0(b);
                this.allocatedMemoryEnabled = b;
            }
        }
        return;
    }
    
    @Override
    public long[] findMonitorDeadlockedThreads() {
        Util.checkMonitorAccess();
        Thread[] monitorDeadlockedThreads0 = findMonitorDeadlockedThreads0();
        if (monitorDeadlockedThreads0 == null) {
            return null;
        }
        long[] array = new long[monitorDeadlockedThreads0.length];
        for (int i = 0; i < monitorDeadlockedThreads0.length; ++i) {
            array[i] = monitorDeadlockedThreads0[i].getId();
        }
        return array;
    }
    
    @Override
    public long[] findDeadlockedThreads() {
        if (!this.isSynchronizerUsageSupported()) {
            throw new UnsupportedOperationException("Monitoring of Synchronizer Usage is not supported.");
        }
        Util.checkMonitorAccess();
        Thread[] deadlockedThreads0 = findDeadlockedThreads0();
        if (deadlockedThreads0 == null) {
            return null;
        }
        long[] array = new long[deadlockedThreads0.length];
        for (int i = 0; i < deadlockedThreads0.length; ++i) {
            array[i] = deadlockedThreads0[i].getId();
        }
        return array;
    }
    
    @Override
    public void resetPeakThreadCount() {
        Util.checkControlAccess();
        resetPeakThreadCount0();
    }
    
    @Override
    public boolean isObjectMonitorUsageSupported() {
        return this.jvm.isObjectMonitorUsageSupported();
    }
    
    @Override
    public boolean isSynchronizerUsageSupported() {
        return this.jvm.isSynchronizerUsageSupported();
    }
    
    private void verifyDumpThreads(boolean b, boolean b2) {
        if (b && !this.isObjectMonitorUsageSupported()) {
            throw new UnsupportedOperationException("Monitoring of Object Monitor Usage is not supported.");
        }
        if (b2 && !this.isSynchronizerUsageSupported()) {
            throw new UnsupportedOperationException("Monitoring of Synchronizer Usage is not supported.");
        }
        Util.checkMonitorAccess();
        return;
    }
    
    @Override
    public ThreadInfo[] getThreadInfo(long[] array, boolean b, boolean b2) {
        this.verifyThreadIds(array);
        if (array.length == 0) {
            return new ThreadInfo[0];
        }
        this.verifyDumpThreads(b, b2);
        return dumpThreads0(array, b, b2);
    }
    
    @Override
    public ThreadInfo[] dumpAllThreads(boolean b, boolean b2) {
        this.verifyDumpThreads(b, b2);
        return dumpThreads0(null, b, b2);
    }
    
    private static native Thread[] getThreads();
    
    private static native void getThreadInfo1(long[] p0, int p1, ThreadInfo[] p2);
    
    private static native long getThreadTotalCpuTime0(long p0);
    
    private static native void getThreadTotalCpuTime1(long[] p0, long[] p1);
    
    private static native long getThreadUserCpuTime0(long p0);
    
    private static native void getThreadUserCpuTime1(long[] p0, long[] p1);
    
    private static native void getThreadAllocatedMemory1(long[] p0, long[] p1);
    
    private static native void setThreadCpuTimeEnabled0(boolean p0);
    
    private static native void setThreadAllocatedMemoryEnabled0(boolean p0);
    
    private static native void setThreadContentionMonitoringEnabled0(boolean p0);
    
    private static native Thread[] findMonitorDeadlockedThreads0();
    
    private static native Thread[] findDeadlockedThreads0();
    
    private static native void resetPeakThreadCount0();
    
    private static native ThreadInfo[] dumpThreads0(long[] p0, boolean p1, boolean p2);
    
    private static native void resetContentionTimes0(long p0);
    
    @Override
    public ObjectName getObjectName() {
        return Util.newObjectName("java.lang:type=Threading");
    }
}
